#!/usr/bin/env python

#
# install this script in /usr/bin/fanservice
# and install the following unit file into /etc/systemd/system
#

import sys

# output the unit file just for grins if we ask for it
if (len(sys.argv) > 1 and sys.argv[1] == 'service'):
    print("""
[Unit]
Description=Fan service
After=multi-user.target
[Service]
Type=simple
Restart=always
RemainAfterExit=true
ExecStart=/usr/bin/fanservice
[Install]
WantedBy=multi-user.target
    """)
    sys.exit(0)

import time
import os
import smbus
import Adafruit_GPIO.I2C as I2C
import subprocess
import math
import Adafruit_SSD1306

from PIL import Image
from PIL import ImageDraw
from PIL import ImageFont

# manifest constants
hat_addr = 0x0d
rgb_effect_reg = 0x04
rgb_off_reg = 0x07
fan_reg = 0x08

# initializers for globals
count = 0
fanspeed = ""
curspeed = 99
display = 0

bus = smbus.SMBus(1)

RST = None     # on the PiOLED this pin isnt used

# 128x32 display with hardware I2C:
disp = Adafruit_SSD1306.SSD1306_128_32(rst=RST)

# Initialize library.
disp.begin()

# Clear display.
disp.clear()
disp.display()

# Make sure to create image with mode '1' for 1-bit color.
width = disp.width
height = disp.height
image = Image.new('1', (width, height))

# Get drawing object to draw on image.
draw = ImageDraw.Draw(image)

fontfile = "/usr/share/fonts/truetype/dejavu/DejaVuSans.ttf"
font1 = ImageFont.truetype(fontfile, 18)
font2 = ImageFont.truetype(fontfile, 14)

#
# fan speeds: 0: off, 1: full, 2: 20%, 3: 30%, ... 9: 90%
#
def setFanSpeed(speed):

    global curspeed
    global fanspeed

    if (speed == 0):
        fanspeed="off"
    elif (speed == 1):
        fanspeed="full"
    else:
        fanspeed=str(speed)+"0%"

    if (curspeed != speed):
        bus.write_byte_data(hat_addr, fan_reg, speed & 0xff)
    curspeed = speed

# turn off the fan for starters
setFanSpeed(0x00)

# we don't need the RGB LED's to burn power
bus.write_byte_data(hat_addr, rgb_off_reg, 0)

while True:
    CPU_TEMP = open("/sys/class/thermal/thermal_zone0/temp", "r").readline();
    temp = math.ceil(int(CPU_TEMP) / 100)/10;

    # Draw a black filled box to clear the image.
    draw.rectangle((0,0,width,height), outline=0, fill=0)

    if (display == 4):
        pass
 
    elif (display == 3):
        LOAD = open("/proc/loadavg", "r").readline().split()
        draw.text((0,0), 
            "load: ",  
            font=font2, fill=255)
        draw.text((8, 16),
            " ".join(LOAD[0:3]),
            font=font2, fill=255)

    elif (display == 2):
        UPTIME = int(open("/proc/uptime", "r").readline().split()[0].split('.')[0])
        days=math.floor(UPTIME/86400)
        plural = "" if (days == 1) else "s"
        hours=math.floor((UPTIME%86400)/3600)
        mins=math.floor((UPTIME%3600)/60)
        secs=math.floor(UPTIME%60)
        draw.text((0,0), 
            "up: ",  
            font=font2, fill=255)
        draw.text((8, 16),
            str(days)+" day"+plural+" "+str(hours)+":"+str(mins)+":"+str(secs),
            font=font2, fill=255)
         
    elif (display == 1):
        IP = os.popen('hostname -I').readline().split()
        IP0 = IP[0]
        IP1 = IP[1]
        draw.text((0, 0), str(IP0),  font=font2, fill=255)
        draw.text((0, 16), str(IP1),  font=font2, fill=255)

    else:
        display=0
        draw.text((0, 8),
            str(temp) + "C " + fanspeed,
            font=font1, fill=255)

    # Display image.
    disp.image(image)
    disp.display()

    display += 1

    if temp >= 60:
        setFanSpeed(1)
    elif temp >= 55:
        setFanSpeed(5)
    elif temp <= 50:
        setFanSpeed(0)
    
    time.sleep(5)

# vim: tabstop=4 softtabstop=4 shiftwidth=4 expandtab
